package com.huimv.env.device.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.env.common.dao.entity.EnvAmmoniaEntity;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.dao.entity.EnvHumiEntity;
import com.huimv.env.common.dao.entity.EnvTempEntity;
import com.huimv.env.common.dao.repo.EnvAmmoniaEntityRepo;
import com.huimv.env.common.dao.repo.EnvHumiEntityRepo;
import com.huimv.env.common.dao.repo.EnvTempEntityRepo;
import com.huimv.env.common.service.IPigpenService;
import com.huimv.env.common.service.ISensorRegisterService;
import com.huimv.env.common.utils.DateUtil;
import com.huimv.env.common.utils.MathUtil;
import com.huimv.env.device.config.ConfigBean;
import com.huimv.env.device.producer.Producer;
import com.huimv.env.device.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Timestamp;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceServiceImpl implements IDeviceService {
    @Autowired
    private EnvTempEntityRepo envTempEntityRepo;
    @Autowired
    private EnvHumiEntityRepo envHumiEntityRepo;
    @Autowired
    private EnvAmmoniaEntityRepo envAmmoniaEntityRepo;
    @Autowired
    private MathUtil mathUtil;
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private Producer producer;
    @Autowired
    private ISensorRegisterService sensorRegisterService;
    @Autowired
    private ConfigBean configBean;
    @Autowired
    private IPigpenService pigpenService;

    @Override
    public void sendSensorToMQ(String deviceCode, Integer sensorSn, Timestamp nowTimestamp, Date todayDate, String value, String farmCode, int sensorSort) {
        JSONObject sensorJo = new JSONObject();
        sensorJo.put("deviceCode", deviceCode);
        sensorJo.put("sensorSn", sensorSn);
        sensorJo.put("timestamp", nowTimestamp);
        sensorJo.put("todayDate", todayDate);
        sensorJo.put("value", value);
        sensorJo.put("farmCode", farmCode);
        sensorJo.put("sensorSort", sensorSort);
        producer.sendSensorAskToMQ(sensorJo);
    }


    //更新传感器数据
    private void updateSensorRegister(String deviceCode, int sensorSn, Timestamp nowTimestamp, Date todayDate, String value, String farmCode, int sensorSort) {
        if (sensorRegisterService.isExistSensorRegister(deviceCode, sensorSn, farmCode, sensorSort)) {
            System.out.println("更新...");
            //{更新传感器注册信息}
            sensorRegisterService.updateSensorRegister(deviceCode, sensorSn, nowTimestamp, todayDate, value, farmCode, sensorSort);
        } else {
            System.out.println("新建...");
            //{保存传感器注册信息}
            sensorRegisterService.saveSensorRegister(deviceCode, sensorSn, nowTimestamp, todayDate, value, farmCode, sensorSort);
        }
    }

    //保存温度流水数据
    @Override
    public void saveTempFlow(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        EnvTempEntity envTempEntity = new EnvTempEntity();
        envTempEntity.setDeviceCode(dataJo.getString("deviceCode"));
        envTempEntity.setSensorSn(dataJo.getInteger("sensorSn"));
        envTempEntity.setTemp(new BigDecimal(mathUtil.countTemp(dataJo.getIntValue("value"))));
        envTempEntity.setAskTime(dateUtil.handleAskTime(dataJo.getString("askTime")));
        envTempEntity.setAddTime(nowTimestamp);
        envTempEntity.setAddDate(todayDate);
        envTempEntity.setFarmCode(farmCode);
        envTempEntity.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envTempEntity.setUnitId(envDeviceRegisterEntity.getUnitId());
        envTempEntity.setDataUnit(dataJo.getString("dataUnit"));
        envTempEntityRepo.saveAndFlush(envTempEntity);
    }

    //保存湿度流水数据
    @Override
    public void saveHumiFlow(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        EnvHumiEntity envHumiEntity = new EnvHumiEntity();
        envHumiEntity.setDeviceCode(dataJo.getString("deviceCode"));
        envHumiEntity.setSensorSn(dataJo.getInteger("sensorSn"));
        envHumiEntity.setHumi(new BigDecimal(mathUtil.countHumi(dataJo.getIntValue("value"))));
        envHumiEntity.setAskTime(dateUtil.handleAskTime(dataJo.getString("askTime")));
        envHumiEntity.setAddTime(nowTimestamp);
        envHumiEntity.setAddDate(todayDate);
        envHumiEntity.setFarmCode(farmCode);
        envHumiEntity.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envHumiEntity.setUnitId(envDeviceRegisterEntity.getUnitId());
        envHumiEntity.setDataUnit(dataJo.getString("dataUnit"));
        envHumiEntityRepo.saveAndFlush(envHumiEntity);
    }

    //保存氨气流水数据
    @Override
    public void saveAmmoniaFlow(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        EnvAmmoniaEntity envAmmoniaEntity = new EnvAmmoniaEntity();
        envAmmoniaEntity.setDeviceCode(dataJo.getString("deviceCode"));
        envAmmoniaEntity.setSensorSn(dataJo.getInteger("sensorSn"));
        envAmmoniaEntity.setAmmonia(new BigDecimal(dataJo.getIntValue("value")));
        envAmmoniaEntity.setAskTime(dateUtil.handleAskTime(dataJo.getString("askTime")));
        envAmmoniaEntity.setAddTime(nowTimestamp);
        envAmmoniaEntity.setAddDate(todayDate);
        envAmmoniaEntity.setFarmCode(farmCode);
        envAmmoniaEntity.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envAmmoniaEntity.setUnitId(envDeviceRegisterEntity.getUnitId());
        envAmmoniaEntity.setDataUnit(dataJo.getString("dataUnit"));
        envAmmoniaEntityRepo.saveAndFlush(envAmmoniaEntity);
    }

    //
    @Override
    public JSONObject handleAskText(String askText) {
        String[] dataArray = askText.split("\\+");
        return buildDataObj(dataArray);
    }

    @Override
    public String getMessageContent(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity) {
        String act = configBean.getAct();
        String fault = configBean.getFault();
        log.info("所有故障类型="+fault);
        log.info("所有动作类型="+act);
        int faultSn = dataJo.getInteger("faultSn");
        int actType = dataJo.getInteger("actType");
        //
        String faultText = getFaultText(faultSn, fault);
        log.info("当前故障="+faultText);
        //1:报警 2:恢复
        String actText = getActText(actType, act);
        log.info("当前操作="+actText);
        String messageConent = "";
        if (actType == 1) {
            if (faultText.indexOf("报警") == -1) {
                messageConent = faultText + actText;
            } else {
                messageConent = faultText;
            }
        } else {
            messageConent = faultText+"已"+actText;
        }
        //获取栋舍
        String pigpenName = pigpenService.getPigpenByDevice(dataJo.getString("deviceCode"));
        //芯片ID
        String chipId = envDeviceRegisterEntity.getChipId();
        //2栋的芯片ID[86178908]高温报警
        return "["+pigpenName+"]芯片ID为["+chipId+"]的设备提示:"+messageConent;
    }

    @Override
    public JSONObject handleSprayStatusAskCmd(String askText) {
        String[] dataArray = askText.split("\\+");
        JSONObject dataJo = new JSONObject();
        dataJo.put("deviceCode",dataArray[1]);
        dataJo.put("sprayStatus",dataArray[3]);
        dataJo.put("askTime",dataArray[4]);
        return dataJo;
    }

    private String getActText(int actType, String act) {
        String[] actGroupArray = act.split(",");
        String actText = "";
        for (int a = 0; a < actGroupArray.length; a++) {
            String actGroup = actGroupArray[a];
            String[] actArray = actGroup.split(":");
            JSONObject newJo = new JSONObject();
            newJo.put("id", actArray[0]);
            newJo.put("fault", actArray[1]);
            if (Integer.parseInt(actArray[0]) == actType) {
                actText = actArray[1];
                break;
            }
        }
        return actText;
    }

    //
    private String getFaultText(int faultSn, String fault) {
        String[] faultGroupArray = fault.split(",");
        String faultText = "";
        for (int a = 0; a < faultGroupArray.length; a++) {
            String falutGroup = faultGroupArray[a];
            String[] faultArray = falutGroup.split(":");
            if (Integer.parseInt(faultArray[0]) == faultSn) {
                faultText = faultArray[1];
                break;
            }
        }
        return faultText;
    }

    //
    public JSONObject buildDataObj(String[] dataArray) {
        JSONObject dataJo = new JSONObject();
        dataJo.put("deviceCode", dataArray[1]);
        dataJo.put("cmd", dataArray[2]);
        dataJo.put("sensorSn", dataArray[3]);
        dataJo.put("value", dataArray[4]);
        dataJo.put("askTime", dataArray[5]);
        return dataJo;
    }

}
